void Players_OnClientPutInServer(int client)
{
	//Player default settings
	b_WaitingForEnemy[client] 	= false;
	i_PlayerArena[client] 		= LOBBY;
	i_PlayerEnemy[client] 		= -1;
	
	//Timers
	SearchTmr[client] 			= null;
	
	//Arena thing settings
	i_PrevArena[client] 		= -1;

}


//**####################**
//~~~ PLAYER DISCONNECT ~~
//**####################**
void Players_OnClientDisconnect(int client)
{
	//Check if player is playing and disconnects
	if(i_PlayerArena[client] != LOBBY)
	{
		//Kill damage timer, because already ingame
		KillDamageTimer(i_PlayerArena[client]);
		
	}
		
	//Print chat message
	int opponent = i_PlayerEnemy[client];
	if(opponent > 0)
	{
		if(IsClientInGame(opponent))
		{
			PrintToChat(opponent, "%sSorry, your enemy left the game!", PREFIX);
			
			i_PrevEnemy[opponent] = -1;
			
			b_ArenaFree[i_PlayerArena[opponent]] = true;
			
			//Teleport to lobby, because enemy left
			TeleportToLobby(opponent, true, true);
			
		
		}
	}
		
}


//**####################**
//~~~ 	PLAYER DEATH 	~~
//**####################**
void Players_OnPlayerDeath(int client, int attacker)
{
	//Someone dies from world?!
	if(attacker == 0) {
		int opponent = i_PlayerEnemy[client];
		if(opponent > 0)
			if(IsClientInGame(opponent))
				SearchTmr[opponent] = CreateTimer(g_DuelDelayCvar.FloatValue, PlayerKilled, opponent);
	}
	
	//Keep them arena for small amount of time, so they know they killed someone :D
	
	if(IsClientInGame(client))
		CreateTimer(0.6, PlayerGotKilled, client);
	
	if(attacker > 0)
		if(IsClientInGame(attacker))
			SearchTmr[attacker] = CreateTimer(g_DuelDelayCvar.FloatValue, PlayerKilled, attacker);
	
}


//----------------------------
//Player who got killed timer
//----------------------------
public Action PlayerGotKilled(Handle tmr, any client)
{
	if(IsClientInGame(client))
		TeleportToLobby(client, true);
}


//------------------------------
//Player who killed other player
//------------------------------
public Action PlayerKilled(Handle tmr, any client)
{
	if(client > 0)
	{
		if(IsClientInGame(client))
		{
			//Destroy timer
			SearchTmr[client] 		 = null;
			
			//Set his arena free
			if(i_PlayerArena[client] != LOBBY)
				b_ArenaFree[i_PlayerArena[client]] = true;
		
			//He wants to find opponent (someone from lobby)
			int opponent = FindOpponent(client);
			if(opponent > -1){
				
				//Setup match
				SetupMatch(client, opponent);
		
			} else {
				//No opponent found
				if(i_PlayerArena[client] != LOBBY)
				{
					PrintToChat(client, "%sSorry, no opponent found!", PREFIX);
					TeleportToLobby(client, true);
				}
				
				//Make him search for enemy, if he can find one
				SearchTmr[client] = CreateTimer(1.0, PlayerKilled, client);
			}
		
		}
	}
}















//**####################**
//~~~ STOCK FUNCTIONS ~~
//**####################**

void TeleportToLobby(int client, bool searchEnable, bool ImSearching = false)
{
	if(client > 0)
	{
		//Set that enemys opponent to -1, because we went to lobby
		int opponent = i_PlayerEnemy[client];
		if(opponent > 0)
			if(IsClientInGame(opponent))
				i_PlayerEnemy[opponent] = -1;
		
		if(IsClientInGame(client))
		{
			
			//Check if player is dead, respawn
			if(!IsPlayerAlive(client))
				CS_RespawnPlayer(client);
			
			float org[3], ang[3];
			GetArenaSpawn(LOBBY, GetClientTeam(client), org, ang);
			TeleportEntity(client, org, ang, NULL_VECTOR);
			i_PlayerArena[client] 		= LOBBY;
			i_PlayerEnemy[client] 		= -1;
			
			//Make 
			SetEntData(client, g_offsCollisionGroup, 2, 4, true);
			
			if(searchEnable)
				b_WaitingForEnemy[client] = true;
			else
				b_WaitingForEnemy[client] = false;
				
			if(ImSearching)
				SearchTmr[client] = CreateTimer(0.1, PlayerKilled, client);
				
		}
	}
}

int FindOpponent(int client)
{
	int opponent = -1;

	//Check if there is any opponent avalibe without last one
	int AllPlayers2[MAXPLAYERS + 1], count2;
	LoopAllPlayers(i)
	{	
		//If the player fits requirements (Is in lobby, has selected weapons, has no opponent, and is waiting for one)
		if(i != client && i_PlayerArena[i] == LOBBY && !b_FirstWeaponSelect[i] && i_PlayerEnemy[i] == -1 && b_WaitingForEnemy[i] && i_PrevEnemy[client] != i)
			AllPlayers2[count2++] = i;
	}
	int opponent2 = (count2 == 0) ? -1 : AllPlayers2[GetRandomInt(0, count2 - 1)];
	

	if(opponent2 == -1)
	{
		
		//If there is no one avalible, try to generate with last enemy also
		int AllPlayers[MAXPLAYERS + 1], count;
		LoopAllPlayers(i)
		{	
			//If the player fits requirements (Is in lobby, has selected weapons, has no opponent, and is waiting for one)
			if(i != client && i_PlayerArena[i] == LOBBY && !b_FirstWeaponSelect[i] && i_PlayerEnemy[i] == -1 && b_WaitingForEnemy[i])
				AllPlayers[count++] = i;
		}
		
			
		//Get one random player from 'AllPlayers'
		opponent = (count == 0) ? -1 : AllPlayers[GetRandomInt(0, count - 1)];
	
	} else {
		
		opponent = opponent2;
		
	}

	
	//Lets check if that player is still avalible
	if(opponent > 0)
	{
		if(b_WaitingForEnemy[opponent] && i_PlayerEnemy[opponent] == -1 && i_PlayerArena[opponent] == LOBBY){
			
			//Set the player not avalible anymore
			b_WaitingForEnemy[client] 		= false;
			b_WaitingForEnemy[opponent] 	= false;
			i_PlayerEnemy[opponent] 		= client;
			i_PlayerEnemy[client] 			= opponent;
			
			return opponent;
		}
	}
	
	return -1;
}

void SetupMatch(int client, int enemy)
{
	
	//Check if both players are still ingame (because of timer bellow)
	if(IsClientInGame(client) && IsClientInGame(enemy))
	{
	
		//Generate free arena, if it didn't find any free arena, try again every 0.1 second
		int arena = GetFreeArena(i_PrevArena[client]);

		if(arena == -1)
		{
			DataPack pack;
			CreateDataTimer(0.1, TrySetupMatchAgain, pack);
			pack.WriteCell(client);
			pack.WriteCell(enemy);
			
			//debug
			int count = 0;
			for (int i = 1; i <= g_maxArenas; i++){
				if(b_ArenaFree[i] && i != LOBBY){
					count++;
				}
			}
			PrintToConsole(client, "%sFree arenas: %i", PREFIX, count);
			PrintToConsole(enemy, "%sFree arenas: %i", PREFIX, count);
			
		} else if(arena > 0) {
			
			//Stop search timers if one of them has one
			KillSearchTimer(client);
			KillSearchTimer(enemy);
			
			//Make the arena not free and set variables
			b_ArenaFree[arena] 		= false;
			i_PlayerArena[client] 	= arena;
			i_PlayerArena[enemy] 	= arena;
			i_PrevArena[client] 	= arena;
			i_PrevArena[enemy] 		= arena;
			i_PrevEnemy[client]	 	= enemy;
			i_PrevEnemy[enemy] 		= client;
			i_DamageGiven[client] 	= 0;
			i_DamageGiven[enemy] 	= 0;
			
			//Setting up players
			SetupPlayer(client, enemy, arena, CS_TEAM_T);
			SetupPlayer(enemy, client, arena, CS_TEAM_CT);	
			
			//Create new duel if there is no damage in 25 seconds
			ArenaDamageTmr[arena] = CreateTimer(g_NoDamageCvar.FloatValue, ArenaDamageTimer, arena);
			
		}
		
	}
}

public Action TrySetupMatchAgain(Handle timer, Handle pack)
{
	ResetPack(pack);
	int client = ReadPackCell(pack);	
	int enemy = ReadPackCell(pack);
	SetupMatch(client, enemy);
}

void KillSearchTimer(int client)
{
	if (SearchTmr[client] != null)
	{
		KillTimer(SearchTmr[client]);
		SearchTmr[client] = null;
	}
}

void KillDamageTimer(int arena)
{
	if (ArenaDamageTmr[arena] != null)
	{
		KillTimer(ArenaDamageTmr[arena]);
		ArenaDamageTmr[arena] = null;
	}
}

void SetupPlayer(int client, int opponent, int arena, int team)
{

	//Teleport to arena
	TeleportToArena(client, team, arena);
	
	//Give right weapons
	if(b_AwpDuelEnabled[client] && b_AwpDuelEnabled[opponent] && g_AWPDuelsCvar.IntValue == 1)
		GiveAWPDuelWeapons(client);
	else
		GivePlayerHisWeapons(client);
	
	//For safety reset health
	SetEntityHealth(client, 100);
	
	//Armor
	if(g_GiveArmorCvar.IntValue == 1)
		SetEntProp(client, Prop_Send, "m_ArmorValue", 100, 10);
	else
		SetEntProp(client, Prop_Send, "m_ArmorValue", 0, 10);
	
	//Helmet
	if(g_GiveHelmetCvar.IntValue == 1)
		SetEntProp(client, Prop_Send, "m_bHasHelmet", 1);
	else
		SetEntProp(client, Prop_Send, "m_bHasHelmet", 0);
	
	
	if(g_ShowUsernameCvar.IntValue == 1)
	{
		//Show username in top left corner
		DataPack pack;
		CreateDataTimer(0.1, DisplayUsername, pack);
		pack.WriteCell(client);
		pack.WriteCell(opponent);
	}
}

public Action DisplayUsername(Handle timer, Handle pack)
{
	ResetPack(pack);
	int client 	= ReadPackCell(pack);
	int client2 = ReadPackCell(pack);
	if(IsClientInGame(client) && IsClientInGame(client2))
	{
		char username[128];
		GetClientName(client2, username, sizeof(username));
		hud_message(client, username);
	}
}


void TeleportToArena(int client, int team, int arena)
{
	float org[3], ang[3], vec[3];
	GetArenaSpawn(arena, team, org, ang);
	TeleportEntity(client, org, ang, vec);
	SetEntData(client, g_offsCollisionGroup, 5, 4, true);
}

public Action ArenaDamageTimer(Handle timer, any arena)
{
	
	//Set timer to null
	ArenaDamageTmr[arena] = null;
	
	//Get both players who are playing in that arena
	int player = -1, player2 = -1;
	LoopAllPlayers(i) 
	{
		if(i_PlayerArena[i] == arena)
		{
			if(player == -1)
				player = i;
			else
				player2 = i;
		}
	}
	
	
	//One player teleport to lobby, one player searches for new enemy
	int random = GetRandomInt(1, 2);
	
	if(random == 1)
	{
		TeleportToLobby(player2, true);
		if(player > 0)
			SearchTmr[player] = CreateTimer(0.1, PlayerKilled, player);
	} else {
		TeleportToLobby(player, true);
		
		if(player2 > 0)
			SearchTmr[player2] = CreateTimer(0.1, PlayerKilled, player2);
	}
	
	b_ArenaFree[arena] = true;
}